@@ -4,7 +4,7 @@ import (
 	"bufio"
 	"context"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"net/textproto"
 	"net/url"
 	"os"
@@ -17,97 +17,100 @@ import (
 )
 
 type ConfigOptions struct {
-	Filter  FilterOptions
-	General GeneralOptions
-	HTTP    HTTPOptions
-	Input   InputOptions
-	Matcher MatcherOptions
-	Output  OutputOptions
+	Filter  FilterOptions  `json:"filters"`
+	General GeneralOptions `json:"general"`
+	HTTP    HTTPOptions    `json:"http"`
+	Input   InputOptions   `json:"input"`
+	Matcher MatcherOptions `json:"matchers"`
+	Output  OutputOptions  `json:"output"`
 }
 
 type HTTPOptions struct {
-	Cookies           []string
-	Data              string
-	FollowRedirects   bool
-	Headers           []string
-	IgnoreBody        bool
-	Method            string
-	ProxyURL          string
-	Recursion         bool
-	RecursionDepth    int
-	RecursionStrategy string
-	ReplayProxyURL    string
-	SNI               string
-	Timeout           int
-	URL               string
-	Http2             bool
+	Cookies           []string `json:"-"` // this is appended in headers
+	Data              string   `json:"data"`
+	FollowRedirects   bool     `json:"follow_redirects"`
+	Headers           []string `json:"headers"`
+	IgnoreBody        bool     `json:"ignore_body"`
+	Method            string   `json:"method"`
+	ProxyURL          string   `json:"proxy_url"`
+	Recursion         bool     `json:"recursion"`
+	RecursionDepth    int      `json:"recursion_depth"`
+	RecursionStrategy string   `json:"recursion_strategy"`
+	ReplayProxyURL    string   `json:"replay_proxy_url"`
+	SNI               string   `json:"sni"`
+	Timeout           int      `json:"timeout"`
+	URL               string   `json:"url"`
+	Http2             bool     `json:"http2"`
 }
 
 type GeneralOptions struct {
-	AutoCalibration         bool
-	AutoCalibrationKeyword  string
-	AutoCalibrationPerHost  bool
-	AutoCalibrationStrategy string
-	AutoCalibrationStrings  []string
-	Colors                  bool
-	ConfigFile              string `toml:"-"`
-	Delay                   string
-	Json                    bool
-	MaxTime                 int
-	MaxTimeJob              int
-	Noninteractive          bool
-	Quiet                   bool
-	Rate                    int
-	ShowVersion             bool `toml:"-"`
-	StopOn403               bool
-	StopOnAll               bool
-	StopOnErrors            bool
-	Threads                 int
-	Verbose                 bool
+	AutoCalibration         bool     `json:"autocalibration"`
+	AutoCalibrationKeyword  string   `json:"autocalibration_keyword"`
+	AutoCalibrationPerHost  bool     `json:"autocalibration_per_host"`
+	AutoCalibrationStrategy string   `json:"autocalibration_strategy"`
+	AutoCalibrationStrings  []string `json:"autocalibration_strings"`
+	Colors                  bool     `json:"colors"`
+	ConfigFile              string   `toml:"-" json:"config_file"`
+	Delay                   string   `json:"delay"`
+	Json                    bool     `json:"json"`
+	MaxTime                 int      `json:"maxtime"`
+	MaxTimeJob              int      `json:"maxtime_job"`
+	Noninteractive          bool     `json:"noninteractive"`
+	Quiet                   bool     `json:"quiet"`
+	Rate                    int      `json:"rate"`
+	ScraperFile             string   `json:"scraperfile"`
+	Scrapers                string   `json:"scrapers"`
+	Searchhash              string   `json:"-"`
+	ShowVersion             bool     `toml:"-" json:"-"`
+	StopOn403               bool     `json:"stop_on_403"`
+	StopOnAll               bool     `json:"stop_on_all"`
+	StopOnErrors            bool     `json:"stop_on_errors"`
+	Threads                 int      `json:"threads"`
+	Verbose                 bool     `json:"verbose"`
 }
 
 type InputOptions struct {
-	DirSearchCompat        bool
-	Extensions             string
-	IgnoreWordlistComments bool
-	InputMode              string
-	InputNum               int
-	InputShell             string
-	Inputcommands          []string
-	Request                string
-	RequestProto           string
-	Wordlists              []string
+	DirSearchCompat        bool     `json:"dirsearch_compat"`
+	Extensions             string   `json:"extensions"`
+	IgnoreWordlistComments bool     `json:"ignore_wordlist_comments"`
+	InputMode              string   `json:"input_mode"`
+	InputNum               int      `json:"input_num"`
+	InputShell             string   `json:"input_shell"`
+	Inputcommands          []string `json:"input_commands"`
+	Request                string   `json:"request_file"`
+	RequestProto           string   `json:"request_proto"`
+	Wordlists              []string `json:"wordlists"`
 }
 
 type OutputOptions struct {
-	DebugLog            string
-	OutputDirectory     string
-	OutputFile          string
-	OutputFormat        string
-	OutputSkipEmptyFile bool
+	DebugLog            string `json:"debug_log"`
+	OutputDirectory     string `json:"output_directory"`
+	OutputFile          string `json:"output_file"`
+	OutputFormat        string `json:"output_format"`
+	OutputSkipEmptyFile bool   `json:"output_skip_empty"`
 }
 
 type FilterOptions struct {
-	Mode   string
-	Lines  string
-	Regexp string
-	Size   string
-	Status string
-	Time   string
-	Words  string
+	Mode   string `json:"mode"`
+	Lines  string `json:"lines"`
+	Regexp string `json:"regexp"`
+	Size   string `json:"size"`
+	Status string `json:"status"`
+	Time   string `json:"time"`
+	Words  string `json:"words"`
 }
 
 type MatcherOptions struct {
-	Mode   string
-	Lines  string
-	Regexp string
-	Size   string
-	Status string
-	Time   string
-	Words  string
+	Mode   string `json:"mode"`
+	Lines  string `json:"lines"`
+	Regexp string `json:"regexp"`
+	Size   string `json:"size"`
+	Status string `json:"status"`
+	Time   string `json:"time"`
+	Words  string `json:"words"`
 }
 
-//NewConfigOptions returns a newly created ConfigOptions struct with default values
+// NewConfigOptions returns a newly created ConfigOptions struct with default values
 func NewConfigOptions() *ConfigOptions {
 	c := &ConfigOptions{}
 	c.Filter.Mode = "or"
@@ -128,6 +131,9 @@ func NewConfigOptions() *ConfigOptions {
 	c.General.Noninteractive = false
 	c.General.Quiet = false
 	c.General.Rate = 0
+	c.General.Searchhash = ""
+	c.General.ScraperFile = ""
+	c.General.Scrapers = "all"
 	c.General.ShowVersion = false
 	c.General.StopOn403 = false
 	c.General.StopOnAll = false
@@ -169,7 +175,7 @@ func NewConfigOptions() *ConfigOptions {
 	return c
 }
 
-//ConfigFromOptions parses the values in ConfigOptions struct, ensures that the values are sane,
+// ConfigFromOptions parses the values in ConfigOptions struct, ensures that the values are sane,
 // and creates a Config struct out of them.
 func ConfigFromOptions(parseOpts *ConfigOptions, ctx context.Context, cancel context.CancelFunc) (*Config, error) {
 	//TODO: refactor in a proper flag library that can handle things like required flags
@@ -220,6 +226,7 @@ func ConfigFromOptions(parseOpts *ConfigOptions, ctx context.Context, cancel con
 		}
 	}
 
+	tmpWordlists := make([]string, 0)
 	for _, v := range parseOpts.Input.Wordlists {
 		var wl []string
 		if runtime.GOOS == "windows" {
@@ -243,6 +250,17 @@ func ConfigFromOptions(parseOpts *ConfigOptions, ctx context.Context, cancel con
 		} else {
 			wl = strings.SplitN(v, ":", 2)
 		}
+		// Try to use absolute paths for wordlists
+		fullpath := ""
+		if wl[0] != "-" {
+			fullpath, err = filepath.Abs(wl[0])
+		} else {
+			fullpath = wl[0]
+		}
+
+		if err == nil {
+			wl[0] = fullpath
+		}
 		if len(wl) == 2 {
 			if conf.InputMode == "sniper" {
 				errs.Add(fmt.Errorf("sniper mode does not support wordlist keywords"))
@@ -261,7 +279,9 @@ func ConfigFromOptions(parseOpts *ConfigOptions, ctx context.Context, cancel con
 				Template: template,
 			})
 		}
+		tmpWordlists = append(tmpWordlists, strings.Join(wl, ":"))
 	}
+	conf.Wordlists = tmpWordlists
 
 	for _, v := range parseOpts.Input.Inputcommands {
 		ic := strings.SplitN(v, ":", 2)
@@ -364,9 +384,9 @@ func ConfigFromOptions(parseOpts *ConfigOptions, ctx context.Context, cancel con
 
 	// Verify proxy url format
 	if len(parseOpts.HTTP.ProxyURL) > 0 {
-		_, err := url.Parse(parseOpts.HTTP.ProxyURL)
-		if err != nil {
-			errs.Add(fmt.Errorf("Bad proxy url (-x) format: %s", err))
+		u, err := url.Parse(parseOpts.HTTP.ProxyURL)
+		if err != nil || u.Opaque != "" || (u.Scheme != "http" && u.Scheme != "https" && u.Scheme != "socks5") {
+			errs.Add(fmt.Errorf("Bad proxy url (-x) format. Expected http, https or socks5 url"))
 		} else {
 			conf.ProxyURL = parseOpts.HTTP.ProxyURL
 		}
@@ -374,9 +394,9 @@ func ConfigFromOptions(parseOpts *ConfigOptions, ctx context.Context, cancel con
 
 	// Verify replayproxy url format
 	if len(parseOpts.HTTP.ReplayProxyURL) > 0 {
-		_, err := url.Parse(parseOpts.HTTP.ReplayProxyURL)
-		if err != nil {
-			errs.Add(fmt.Errorf("Bad replay-proxy url (-replay-proxy) format: %s", err))
+		u, err := url.Parse(parseOpts.HTTP.ReplayProxyURL)
+		if err != nil || u.Opaque != "" || (u.Scheme != "http" && u.Scheme != "https" && u.Scheme != "socks5" && u.Scheme != "socks5h") {
+			errs.Add(fmt.Errorf("Bad replay-proxy url (-replay-proxy) format. Expected http, https or socks5 url"))
 		} else {
 			conf.ReplayProxyURL = parseOpts.HTTP.ReplayProxyURL
 		}
@@ -446,6 +466,8 @@ func ConfigFromOptions(parseOpts *ConfigOptions, ctx context.Context, cancel con
 	conf.OutputSkipEmptyFile = parseOpts.Output.OutputSkipEmptyFile
 	conf.IgnoreBody = parseOpts.HTTP.IgnoreBody
 	conf.Quiet = parseOpts.General.Quiet
+	conf.ScraperFile = parseOpts.General.ScraperFile
+	conf.Scrapers = parseOpts.General.Scrapers
 	conf.StopOn403 = parseOpts.General.StopOn403
 	conf.StopOnAll = parseOpts.General.StopOnAll
 	conf.StopOnErrors = parseOpts.General.StopOnErrors
@@ -530,11 +552,12 @@ func ConfigFromOptions(parseOpts *ConfigOptions, ctx context.Context, cancel con
 	if parseOpts.General.Verbose && parseOpts.General.Json {
 		errs.Add(fmt.Errorf("Cannot have -json and -v"))
 	}
-
 	return &conf, errs.ErrorOrNil()
 }
 
 func parseRawRequest(parseOpts *ConfigOptions, conf *Config) error {
+	conf.RequestFile = parseOpts.Input.Request
+	conf.RequestProto = parseOpts.Input.RequestProto
 	file, err := os.Open(parseOpts.Input.Request)
 	if err != nil {
 		return fmt.Errorf("could not open request file: %s", err)
@@ -589,7 +612,7 @@ func parseRawRequest(parseOpts *ConfigOptions, conf *Config) error {
 	}
 
 	// Set the request body
-	b, err := ioutil.ReadAll(r)
+	b, err := io.ReadAll(r)
 	if err != nil {
 		return fmt.Errorf("could not read request body: %s", err)
 	}
@@ -669,7 +692,7 @@ func templatePresent(template string, conf *Config) bool {
 
 func ReadConfig(configFile string) (*ConfigOptions, error) {
 	conf := NewConfigOptions()
-	configData, err := ioutil.ReadFile(configFile)
+	configData, err := os.ReadFile(configFile)
 	if err == nil {
 		err = toml.Unmarshal(configData, conf)
 	}
@@ -677,10 +700,14 @@ func ReadConfig(configFile string) (*ConfigOptions, error) {
 }
 
 func ReadDefaultConfig() (*ConfigOptions, error) {
-	userhome, err := os.UserHomeDir()
-	if err != nil {
-		return NewConfigOptions(), err
+	// Try to create configuration directory, ignore the potential error
+	_ = CheckOrCreateConfigDir()
+	conffile := filepath.Join(CONFIGDIR, "ffufrc")
+	if !FileExists(conffile) {
+		userhome, err := os.UserHomeDir()
+		if err == nil {
+			conffile = filepath.Join(userhome, ".ffufrc")
+		}
 	}
-	defaultconf := filepath.Join(userhome, ".ffufrc")
-	return ReadConfig(defaultconf)
+	return ReadConfig(conffile)
 }
