@@ -4,9 +4,9 @@ import (
 	"crypto/md5"
 	"encoding/json"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"path"
+	"sort"
 	"strconv"
 	"strings"
 	"time"
@@ -28,6 +28,7 @@ const (
 
 type Stdoutput struct {
 	config         *ffuf.Config
+	fuzzkeywords   []string
 	Results        []ffuf.Result
 	CurrentResults []ffuf.Result
 }
@@ -37,6 +38,11 @@ func NewStdoutput(conf *ffuf.Config) *Stdoutput {
 	outp.config = conf
 	outp.Results = make([]ffuf.Result, 0)
 	outp.CurrentResults = make([]ffuf.Result, 0)
+	outp.fuzzkeywords = make([]string, 0)
+	for _, ip := range conf.InputProviders {
+		outp.fuzzkeywords = append(outp.fuzzkeywords, ip.Keyword)
+	}
+	sort.Strings(outp.fuzzkeywords)
 	return &outp
 }
 
@@ -324,6 +330,7 @@ func (s *Stdoutput) Result(resp ffuf.Response) {
 		ContentLines:     resp.ContentLines,
 		ContentType:      resp.ContentType,
 		RedirectLocation: resp.GetRedirectLocation(false),
+		ScraperData:      resp.ScraperData,
 		Url:              resp.Request.Url,
 		Duration:         resp.Time,
 		ResultFile:       resp.ResultFile,
@@ -352,7 +359,7 @@ func (s *Stdoutput) writeResultToFile(resp ffuf.Response) string {
 	fileName = fmt.Sprintf("%x", md5.Sum([]byte(fileContent)))
 
 	filePath = path.Join(s.config.OutputDirectory, fileName)
-	err := ioutil.WriteFile(filePath, []byte(fileContent), 0640)
+	err := os.WriteFile(filePath, []byte(fileContent), 0640)
 	if err != nil {
 		s.Error(err.Error())
 	}
@@ -361,11 +368,11 @@ func (s *Stdoutput) writeResultToFile(resp ffuf.Response) string {
 
 func (s *Stdoutput) PrintResult(res ffuf.Result) {
 	switch {
-	case s.config.Quiet:
-		s.resultQuiet(res)
 	case s.config.Json:
 		s.resultJson(res)
-	case len(res.Input) > 1 || s.config.Verbose || len(s.config.OutputDirectory) > 0:
+	case s.config.Quiet:
+		s.resultQuiet(res)
+	case len(res.Input) > 1 || s.config.Verbose || len(s.config.OutputDirectory) > 0 || len(res.ScraperData) > 0:
 		// Print a multi-line result (when using multiple input keywords and wordlists)
 		s.resultMultiline(res)
 	default:
@@ -377,7 +384,7 @@ func (s *Stdoutput) prepareInputsOneLine(res ffuf.Result) string {
 	inputs := ""
 	if len(res.Input) > 1 {
 		for k, v := range res.Input {
-			if inSlice(k, s.config.CommandKeywords) {
+			if ffuf.StrInSlice(k, s.config.CommandKeywords) {
 				// If we're using external command for input, display the position instead of input
 				inputs = fmt.Sprintf("%s%s : %s ", inputs, k, strconv.Itoa(res.Position))
 			} else {
@@ -386,7 +393,7 @@ func (s *Stdoutput) prepareInputsOneLine(res ffuf.Result) string {
 		}
 	} else {
 		for k, v := range res.Input {
-			if inSlice(k, s.config.CommandKeywords) {
+			if ffuf.StrInSlice(k, s.config.CommandKeywords) {
 				// If we're using external command for input, display the position instead of input
 				inputs = strconv.Itoa(res.Position)
 			} else {
@@ -416,13 +423,21 @@ func (s *Stdoutput) resultMultiline(res ffuf.Result) {
 	if res.ResultFile != "" {
 		reslines = fmt.Sprintf("%s%s| RES | %s\n", reslines, TERMINAL_CLEAR_LINE, res.ResultFile)
 	}
-	for k, v := range res.Input {
-		if inSlice(k, s.config.CommandKeywords) {
+	for _, k := range s.fuzzkeywords {
+		if ffuf.StrInSlice(k, s.config.CommandKeywords) {
 			// If we're using external command for input, display the position instead of input
 			reslines = fmt.Sprintf(res_str, reslines, TERMINAL_CLEAR_LINE, k, strconv.Itoa(res.Position))
 		} else {
 			// Wordlist input
-			reslines = fmt.Sprintf(res_str, reslines, TERMINAL_CLEAR_LINE, k, v)
+			reslines = fmt.Sprintf(res_str, reslines, TERMINAL_CLEAR_LINE, k, res.Input[k])
+		}
+	}
+	if len(res.ScraperData) > 0 {
+		reslines = fmt.Sprintf("%s%s| SCR |\n", reslines, TERMINAL_CLEAR_LINE)
+		for k, vslice := range res.ScraperData {
+			for _, v := range vslice {
+				reslines = fmt.Sprintf(res_str, reslines, TERMINAL_CLEAR_LINE, k, v)
+			}
 		}
 	}
 	fmt.Printf("%s\n%s\n", res_hdr, reslines)
@@ -466,12 +481,3 @@ func (s *Stdoutput) colorize(status int64) string {
 func printOption(name []byte, value []byte) {
 	fmt.Fprintf(os.Stderr, " :: %-16s : %s\n", name, value)
 }
-
-func inSlice(key string, slice []string) bool {
-	for _, v := range slice {
-		if v == key {
-			return true
-		}
-	}
-	return false
-}
